"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const oas3_chow_chow_1 = require("oas3-chow-chow");
exports.ChowError = oas3_chow_chow_1.ChowError;
exports.RequestValidationError = oas3_chow_chow_1.RequestValidationError;
exports.ResponseValidationError = oas3_chow_chow_1.ResponseValidationError;
const openapi_ui_1 = require("./openapi-ui");
const jsonfile = require("jsonfile");
const yaml = require("js-yaml");
const fs = require("fs");
const oasValidator = require("oas-validator");
function oas(cfg) {
    const config = config_1.validateConfig(cfg);
    const { compiled, doc } = compileOas(config);
    const mw = async (ctx, next) => {
        if (ctx.path === config.endpoint) {
            ctx.body = doc;
            return;
        }
        if (ctx.path === config.uiEndpoint) {
            ctx.body = openapi_ui_1.openapiUI({
                title: doc.info ? doc.info.title : 'openapi UI',
                url: config.endpoint,
                swaggerUiBundleBasePath: config.swaggerUiBundleBasePath
            });
            return;
        }
        if (!config.validatePaths.some(path => ctx.path.startsWith(path))) {
            // Skip validation if no path matches
            return await next();
        }
        try {
            const validRequest = compiled.validateRequest(ctx.path, {
                method: ctx.request.method,
                header: ctx.request.header,
                query: ctx.request.query,
                path: ctx.params,
                cookie: ctx.cookies,
                body: ctx.request.body,
            });
            // Store coerced values
            ctx.oas = {
                request: {
                    query: validRequest.query,
                    params: validRequest.path && validRequest.path.params,
                    header: validRequest.header
                }
            };
        }
        catch (err) {
            config.errorHandler(err, ctx);
        }
        await next();
        if (config.validateResponse) {
            try {
                compiled.validateResponse(ctx.path, {
                    method: ctx.method,
                    status: ctx.status,
                    header: ctx.response.header,
                    body: ctx.body
                });
            }
            catch (err) {
                config.errorHandler(err, ctx);
            }
        }
    };
    return mw;
}
exports.oas = oas;
function loadFromFile(file) {
    if (!file) {
        throw new Error("Missing file path");
    }
    switch (true) {
        case file.endsWith('.json'): {
            return jsonfile.readFileSync(file);
        }
        case file.endsWith('.yml') || file.endsWith('.yaml'): {
            return yaml.safeLoad(fs.readFileSync(file, 'utf8'));
        }
        default:
            throw new Error('Unsupported file format');
    }
}
function compileOas(config) {
    let openApiObject = config.spec || loadFromFile(config.file);
    if (!oasValidator.validateSync(openApiObject, {})) {
        throw new Error('Invalid Openapi document');
    }
    return {
        compiled: new oas3_chow_chow_1.default(openApiObject),
        doc: openApiObject,
    };
}
//# sourceMappingURL=index.js.map